
import argparse
import ftplib
from urllib.parse import urlparse
import os.path
import sys

def main(args):
    url = args.url
    outdir = args.outdir
    user = None
    password = None
    if args.user is not None:
        user = args.user
    if args.password is not None:
        password = args.password

    # parse the url
    parts = urlparse(url)

    # connect to host, default port
    ftp = ftplib.FTP(host=parts.netloc, user=user, passwd=password)
    try:
        ftp.login()
    except Exception as e:
        if str(e).find('503') >= 0:
            pass
        else:
            print('Could not login to FTP server: Error "%s"' % (str(e)))

    try:
        ftp.cwd(parts.path)  # this will fail if the path is a file
        # get the list of files
        filenames = ftp.nlst()
        for fname in filenames:
            print('Fetching %s...' % fname)
            localfile = os.path.join(outdir, fname)
            f = open(localfile, 'wb')
            ftp.retrbinary('RETR %s' % fname, f.write)
        ftp.close()
    except Exception as e:
        path, fname = os.path.split(parts.path)
        filenames = [fname]
        try:
            ftp.cwd(path)
            localfile = os.path.join(outdir, fname)
            f = open(localfile, 'wb')
            print('Fetching %s...' % fname)
            ftp.retrbinary('RETR %s' % fname, f.write)
        except Exception as e:
            ftp.close()
            print('Could not resolve path or file name. Exiting.')
            sys.exit(1)

    print('Downloaded %i files to %s' % (len(filenames), outdir))


if __name__ == '__main__':
    desc = '''Fetch a file or directory of files from strong motion FTP site.

Examples: 
    
Getting a tarball file from the KNET FTP site (with appropriate username/password):

%(prog)s -u user -p passwd ftp://www.k-net.bosai.go.jp/knet/alldata/2018/01/20180124195100.knt.tar.gz ~/tmp

Getting all data files from an event on the GeoNet FTP site:

%(prog)s ftp://ftp.geonet.org.nz/strong/processed/Proc/2018/02_Feb/2018-02-12_211554/Vol2/data/ ~/tmp

    '''
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('url', help='FTP url.')
    parser.add_argument('outdir', help='Output data directory.')

    parser.add_argument('-u', '--user', help='FTP user name.')
    parser.add_argument('-p', '--password', help='FTP password.')

    pargs = parser.parse_args()
    main(pargs)
